package org.teiid.translator.cleaner;

import org.teiid.language.Argument;
import org.teiid.language.Call;
import org.teiid.language.Command;
import org.teiid.language.QueryExpression;
import org.teiid.metadata.RuntimeMetadata;
import org.teiid.translator.*;

import java.util.List;


/**
 *Delegating Translator for cleaning data
 *Teiid 8.4 CR2
 *June 12, 2013 
 **/

@Translator(name="cleaner")
public class cleaner extends BaseDelegatingExecutionFactory<Object, Object> {
	
@Override
	public Execution createExecution(Command command, ExecutionContext executionContext, RuntimeMetadata metadata, Object connection) throws TranslatorException {
		if (command instanceof Call) {
			Call obj = (Call)command;
			//TODO: our extension property support in designer makes ad-hoc properties impossible, so
			//we just match based upon name. it would be better to have the metadatarepository/tooling add a teiid property
			//to explicitly set this proc as direct.
			//the other approach would be to addd a native system stored procedure, but that would require
			//special security semantics, whereas this proc can be secured on a schema basis
			if (supportsNativeQueries() && obj.getMetadataObject().getName().equals(getNativeQueryProcedureName())) {
				List<Argument> arguments = obj.getArguments();
				return createDirectExecution(arguments, command, executionContext, metadata, connection);
			}
		}
		if (command instanceof QueryExpression) {
			return createResultSetExecution((QueryExpression)command, executionContext, metadata, connection);
		}
		if (command instanceof Call) {
			return createProcedureExecution((Call)command, executionContext, metadata, connection);
		}
		return createUpdateExecution(command, executionContext, metadata, connection);
	}
	
	
    @Override
    public ResultSetExecution createResultSetExecution(QueryExpression command, ExecutionContext executionContext, RuntimeMetadata metadata, Object connection) throws TranslatorException {
    	

		final ResultSetExecution rse = super.createResultSetExecution(command, executionContext, metadata, connection);

        return new ResultSetExecution() {

            /*
             * Returning the next row of results into a list
             * First iteration, printing out the index of item "new york" in the list
             */

          

            @Override
            public void close() {
                rse.close();
            }

            @Override
            public void cancel() throws TranslatorException {
                rse.cancel();

            }

            @Override
            public void execute() throws TranslatorException {
                rse.execute();
            }

			@Override
			public List<?> next() throws TranslatorException, DataNotAvailableException {
				//List<?> row = rse.next();
				@SuppressWarnings("unchecked")
				List<String> row = (List<String>) rse.next();
				
				//if(row.isEmpty()){
					//DO Nothing
				//}
								
				//else if (row.contains("nyc")){
					//int index = 0;
					//index = row.indexOf("nyc");
					//row.set(index, "New York");
				//}
				
								
				return row;
			}
        };

      	
        


    }
    
   	
}
